# coding=utf-8
from __future__ import division, print_function, unicode_literals

from django.contrib import admin
from django.db.models import Avg, Count
from django.db.models.functions import Coalesce

from admin_totals.admin import ModelAdminTotals

from .models import Band

site = admin.AdminSite(name="admin")


class BandAdmin(ModelAdminTotals):
    list_display = ['name', 'nr_of_members', '_genres']
    list_totals = [
        ('nr_of_members', Avg),
        ('_genres', lambda field: Coalesce(Count('genres'), 0))
    ]
    ordering = ('nr_of_members',)

    def _genres(self, band):
        return ', '.join(str(g) for g in band.genres.all())


site.register(Band, BandAdmin)



# coding=utf-8
from __future__ import division, print_function, unicode_literals

from django.contrib.admin.templatetags.admin_list import (result_headers,
                                                          result_hidden_fields,
                                                          results)
from django.template import Library

register = Library()


@register.inclusion_tag("admin_totals/change_list_results_totals.html")
def result_list_totals(cl):
    """
    Displays the headers, totals bar and data list together
    """
    headers = list(result_headers(cl))
    num_sorted_fields = 0
    for h in headers:
        if h['sortable'] and h['sorted']:
            num_sorted_fields += 1
    return {'cl': cl,
            'result_hidden_fields': list(result_hidden_fields(cl)),
            'result_headers': headers,
            'num_sorted_fields': num_sorted_fields,
            'results': list(results(cl))}



# coding=utf-8
from __future__ import division, print_function, unicode_literals

from django.contrib import admin
from django.contrib.admin.views.main import ChangeList


class ChangeListTotals(ChangeList):
    def get_results(self, *args, **kwargs):
        super(ChangeListTotals, self).get_results(*args, **kwargs)
        if hasattr(self.model_admin, 'list_totals'):
            self.aggregations = []
            list_totals = dict(self.model_admin.list_totals)
            for field in self.list_display:
                if field in list_totals:
                    self.aggregations.append(
                        self.result_list.aggregate(agg=list_totals[field](field))['agg'])
                else:
                    self.aggregations.append('')


class ModelAdminTotals(admin.ModelAdmin):
    change_list_template = 'admin_totals/change_list_totals.html'

    def get_changelist(self, request, **kwargs):
        return ChangeListTotals



# coding=utf-8
from __future__ import division, print_function, unicode_literals

from django.db import models


class Genre(models.Model):
    name = models.CharField(max_length=20)

    class Meta:
        ordering = ('name',)

    def __str__(self):
        return self.name


class Band(models.Model):
    name = models.CharField(max_length=20)
    nr_of_members = models.PositiveIntegerField()
    genres = models.ManyToManyField(Genre)

    def __str__(self):
        return self.name



# coding=utf-8
from __future__ import division, print_function, unicode_literals

from distutils.version import LooseVersion

import django
from django.contrib.auth.models import User
from django.test import TestCase, override_settings
from django.test.client import RequestFactory
from django.urls import reverse
from django.utils.html import strip_spaces_between_tags

from .admin import BandAdmin
from .admin import site as custom_site
from .models import Band

# Test code based on Django's own test code: django/tests/admin_changelist/tests.py


def get_changelist_args(modeladmin, **kwargs):
    m = modeladmin
    args = (
        kwargs.pop('list_display', m.list_display),
        kwargs.pop('list_display_links', m.list_display_links),
        kwargs.pop('list_filter', m.list_filter),
        kwargs.pop('date_hierarchy', m.date_hierarchy),
        kwargs.pop('search_fields', m.search_fields),
        kwargs.pop('list_select_related', m.list_select_related),
        kwargs.pop('list_per_page', m.list_per_page),
        kwargs.pop('list_max_show_all', m.list_max_show_all),
        kwargs.pop('list_editable', m.list_editable),
        m,
    )
    assert not kwargs, "Unexpected kwarg %s" % kwargs
    return args


@override_settings(ROOT_URLCONF="tests.admin_changelist.urls")
class ChangeListTests(TestCase):
    factory = RequestFactory()

    @classmethod
    def setUpTestData(cls):
        cls.superuser = User.objects.create_superuser(
            username='super', email='a@b.com', password='xxx')
        pink_floyd = Band.objects.create(name='Pink Floyd', nr_of_members=5)
        pink_floyd.genres.create(name='Progressive rock')
        pink_floyd.genres.create(name='Psychedelic rock')
        blondie = Band.objects.create(name='Blondie', nr_of_members=6)
        blondie.genres.create(name='New wave')
        blondie.genres.create(name='Pop rock')
        blondie.genres.create(name='Punk rock')

    def setUp(self):
        self.client.force_login(self.superuser)

    def _mocked_authenticated_request(self, url, user):
        request = self.factory.get(url)
        request.user = user
        return request

    def _test_aggregations_django1(self):
        band_admin = BandAdmin(Band, custom_site)
        request = self._mocked_authenticated_request('/band/', self.superuser)
        ChangeList = band_admin.get_changelist(request)
        cl = ChangeList(request, Band, *get_changelist_args(band_admin))
        cl.get_results(request)
        self.assertEqual(
            cl.aggregations,
            [
                '',
                5.5,
                5
            ]
        )

    def _test_aggregations_django2(self):
        band_admin = BandAdmin(Band, custom_site)
        request = self._mocked_authenticated_request('/band/', self.superuser)
        cl = band_admin.get_changelist_instance(request)
        cl.get_results(request)
        self.assertEqual(
            cl.aggregations,
            [
                '',
                '',
                5.5,
                5
            ]
        )

    def test_aggregations(self):
        if LooseVersion(django.__version__) < LooseVersion('2'):
            self._test_aggregations_django1()
        else:
            self._test_aggregations_django2()

    def _test_template_django1(self):
        response = self.client.get(reverse('admin:admin_changelist_band_changelist'))
        content_row1_html = '''
            <tr class="row1">
            <td class="action-checkbox">
            <input type="checkbox" name="_selected_action" value="1" class="action-select" />
            </td>
            <th class="field-name">
            <a href="/admin/admin_changelist/band/1/change/">Pink Floyd</a>
            </th>
            <td class="field-nr_of_members">5</td>
            <td class="field-_genres">Progressive rock, Psychedelic rock</td>
            </tr>
        '''
        content_row2_html = '''
            <tr class="row2">
            <td class="action-checkbox">
            <input type="checkbox" name="_selected_action" value="2" class="action-select" />
            </td>
            <th class="field-name">
            <a href="/admin/admin_changelist/band/2/change/">Blondie</a>
            </th>
            <td class="field-nr_of_members">6</td>
            <td class="field-_genres">New wave, Pop rock, Punk rock</td>
            </tr>
        '''
        totals_row_html = '''
            <tr>
            <th>
              <b style="padding: 8px;"></b>
            </th>
            <th>
              <b style="padding: 8px;"></b>
            </th>
            <th>
              <b style="padding: 8px;">5.5</b>
            </th>
            <th>
              <b style="padding: 8px;">5</b>
            </th>
            </tr>
        '''
        response_html = strip_spaces_between_tags(response.rendered_content)
        for expected_element_html in [
            content_row1_html,
            content_row2_html,
            totals_row_html
        ]:
            expected_element_html = strip_spaces_between_tags(expected_element_html).strip()
            self.assertNotEqual(
                response_html.find(expected_element_html),
                -1,
                'Failed to find expected row element: %s' % expected_element_html)

    def _test_template_django2(self):
        response = self.client.get(reverse('admin:admin_changelist_band_changelist'))
        content_row1_html = '''
            <tr class="row1">
            <td class="action-checkbox">
            <input type="checkbox" name="_selected_action" value="1" class="action-select">
            </td>
            <th class="field-name">
            <a href="/admin/admin_changelist/band/1/change/">Pink Floyd</a>
            </th>
            <td class="field-nr_of_members">5</td>
            <td class="field-_genres">Progressive rock, Psychedelic rock</td>
            </tr>
        '''
        content_row2_html = '''
            <tr class="row2">
            <td class="action-checkbox">
            <input type="checkbox" name="_selected_action" value="2" class="action-select">
            </td>
            <th class="field-name">
            <a href="/admin/admin_changelist/band/2/change/">Blondie</a>
            </th>
            <td class="field-nr_of_members">6</td>
            <td class="field-_genres">New wave, Pop rock, Punk rock</td>
            </tr>
        '''
        totals_row_html = '''
            <tr>
            <th>
              <b style="padding: 8px;"></b>
            </th>
            <th>
              <b style="padding: 8px;"></b>
            </th>
            <th>
              <b style="padding: 8px;">5.5</b>
            </th>
            <th>
              <b style="padding: 8px;">5</b>
            </th>
            </tr>
        '''
        response_html = strip_spaces_between_tags(response.rendered_content)
        for expected_element_html in [
            content_row1_html,
            content_row2_html,
            totals_row_html
        ]:
            expected_element_html = strip_spaces_between_tags(expected_element_html).strip()
            self.assertNotEqual(
                response_html.find(expected_element_html),
                -1,
                'Failed to find expected row element: %s' % expected_element_html)

    def test_template(self):
        if LooseVersion(django.__version__) < LooseVersion('2.1'):
            # Django 2.0 passes only with this test
            self._test_template_django1()
        else:
            self._test_template_django2()



# coding=utf-8
from __future__ import division, print_function, unicode_literals

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    },
}

INSTALLED_APPS = (
    'django.contrib.contenttypes',
    'django.contrib.auth',
    'django.contrib.sites',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.admin.apps.SimpleAdminConfig',
    'django.contrib.staticfiles',

    'admin_totals',

    'tests.admin_changelist',
)

MIDDLEWARE = [
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
]

ROOT_URLCONF = 'tests.urls'

USE_TZ = True

SECRET_KEY = 'foobar'

TEMPLATES = [{
    'BACKEND': 'django.template.backends.django.DjangoTemplates',
    'APP_DIRS': True,
    'OPTIONS': {
        'context_processors': [
            'django.template.context_processors.debug',
            'django.template.context_processors.request',
            'django.contrib.auth.context_processors.auth',
            'django.contrib.messages.context_processors.messages',
        ],
    },
}]

STATIC_URL = '/static/'



from setuptools import setup, find_packages

REQUIREMENTS = (
    'Django',
)
TEST_REQUIREMENTS = (
    'markdown==2.6.4',
    'coverage',
    'flake8',
    'isort',
    'mock',
    'pytz',
    'unittest-xml-reporting',
)


setup(
    name="admin-totals",
    version='1.0.1',
    author="Douwe van der Meij",
    author_email="douwe@karibu-online.nl",
    description="""Django Admin Totals, add totals to your columns in Django admin.
    """,
    long_description=open('README.md', 'rt').read(),
    long_description_content_type="text/markdown",
    url="https://github.com/douwevandermeij/admin-totals",
    packages=find_packages(),
    include_package_data=True,
    install_requires=REQUIREMENTS,
    tests_require=TEST_REQUIREMENTS,
    zip_safe=False,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'License :: OSI Approved :: GNU Lesser General Public License v2 or later (LGPLv2+)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ]
)
